//    |  /           |
//    ' /   __| _` | __|  _ \   __|
//    . \  |   (   | |   (   |\__ \.
//   _|\_\_|  \__,_|\__|\___/ ____/
//                   Multi-Physics
//
//  License:		 BSD License
//					 Kratos default license: kratos/license.txt
//
//  Main authors:    Riccardo Rossi
//

// System includes
#include <string>
#include <iostream>
#include <vector>

// External includes

// Project includes
#include "includes/define.h"
#include "includes/c2c_variables.h"
#include "includes/kernel.h"
#include "includes/node.h"

#include "includes/kratos_flags.h"

namespace Kratos
{

    KRATOS_CREATE_VARIABLE( double, SOLID_TEMPERATURE )
    KRATOS_CREATE_VARIABLE( double, FLUID_TEMPERATURE )
    KRATOS_CREATE_VARIABLE( double, AVERAGE_TEMPERATURE )
    KRATOS_CREATE_VARIABLE( double, INLET_TEMPERATURE)

    KRATOS_CREATE_VARIABLE(double, LAST_AIR)
    KRATOS_CREATE_VARIABLE(double, PRESSURES) 

    // for Vulcan application virtual mould properties
    KRATOS_CREATE_VARIABLE(double,  MOLD_DENSITY)
    KRATOS_CREATE_VARIABLE(double,  MOLD_SPECIFIC_HEAT)
    KRATOS_CREATE_VARIABLE(double,  MOLD_THICKNESS)
    KRATOS_CREATE_VARIABLE(double,  MOLD_SFACT)
    KRATOS_CREATE_VARIABLE(double,  MOLD_VFACT)
    KRATOS_CREATE_VARIABLE(double,  MOLD_CONDUCTIVITY)
    KRATOS_CREATE_VARIABLE(double,  MOLD_HTC_ENVIRONMENT)
    KRATOS_CREATE_VARIABLE(double,  MOLD_TEMPERATURE)
    KRATOS_CREATE_VARIABLE(double,  MOLD_INNER_TEMPERATURE)
    // for Click2Cast Application
    
    KRATOS_CREATE_VARIABLE(double,  HTC)
    KRATOS_CREATE_VARIABLE(double, SOLIDFRACTION);
	KRATOS_CREATE_VARIABLE(double, SOLIDFRACTION_RATE)
    KRATOS_CREATE_VARIABLE(double, SOLIDIF_TIME)
    KRATOS_CREATE_VARIABLE(double, SOLIDIF_MODULUS)
    KRATOS_CREATE_VARIABLE(double, FILLTIME)
    KRATOS_CREATE_VARIABLE(double, MACRO_POROSITY )
    KRATOS_CREATE_VARIABLE(double, SHRINKAGE_POROSITY)
    KRATOS_CREATE_VARIABLE(double, MAX_VEL)
    KRATOS_CREATE_VARIABLE(double, EPSILON )
    KRATOS_CREATE_VARIABLE(double, FRONT_MEETING)
    KRATOS_CREATE_VARIABLE( double, MOLD_AVERAGE_TEMPERATURE )
    KRATOS_CREATE_VARIABLE(double, LIQUID_TIME)
    KRATOS_CREATE_VARIABLE(double, FLOW_LENGTH)
    KRATOS_CREATE_VARIABLE(double, FLOW_LENGTH2)
    KRATOS_CREATE_VARIABLE(double, COOLING_RATE)
    KRATOS_CREATE_VARIABLE(double, LIQUID_TO_SOLID_TIME);
	KRATOS_CREATE_VARIABLE( double, TIME_CRT )
	KRATOS_CREATE_VARIABLE(double, SINKMARK)
	KRATOS_CREATE_VARIABLE(double, COLD_SHUTS)
    KRATOS_CREATE_VARIABLE(double, NIYAMA)
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS (TEMPERATURE_GRADIENT)
    KRATOS_CREATE_VARIABLE(double, INITIAL_TEMPERATURE)

  void KratosApplication::RegisterC2CVariables()
  {
        KRATOS_REGISTER_VARIABLE(  SOLID_TEMPERATURE )
        KRATOS_REGISTER_VARIABLE(  FLUID_TEMPERATURE )
        KRATOS_REGISTER_VARIABLE(  AVERAGE_TEMPERATURE )
        KRATOS_REGISTER_VARIABLE(  INLET_TEMPERATURE)
        
        KRATOS_REGISTER_VARIABLE(  MOLD_AVERAGE_TEMPERATURE)
        KRATOS_REGISTER_VARIABLE(   LAST_AIR)
        KRATOS_REGISTER_VARIABLE(   PRESSURES)

        KRATOS_REGISTER_VARIABLE(   MOLD_DENSITY)
        KRATOS_REGISTER_VARIABLE(   MOLD_SPECIFIC_HEAT)
        KRATOS_REGISTER_VARIABLE(   MOLD_THICKNESS)
        KRATOS_REGISTER_VARIABLE(   MOLD_SFACT)
        KRATOS_REGISTER_VARIABLE(   MOLD_VFACT)
        KRATOS_REGISTER_VARIABLE(   MOLD_CONDUCTIVITY)
        KRATOS_REGISTER_VARIABLE(   MOLD_HTC_ENVIRONMENT)
        KRATOS_REGISTER_VARIABLE(   MOLD_TEMPERATURE)
        KRATOS_REGISTER_VARIABLE(   MOLD_INNER_TEMPERATURE)

        KRATOS_REGISTER_VARIABLE(   HTC)
        KRATOS_REGISTER_VARIABLE(  SOLIDFRACTION )
		KRATOS_REGISTER_VARIABLE( SOLIDFRACTION_RATE)
        KRATOS_REGISTER_VARIABLE(  SOLIDIF_TIME  )
        KRATOS_REGISTER_VARIABLE(  SOLIDIF_MODULUS  )
        KRATOS_REGISTER_VARIABLE(  FILLTIME  )
        KRATOS_REGISTER_VARIABLE(  MACRO_POROSITY  )
        KRATOS_REGISTER_VARIABLE(  SHRINKAGE_POROSITY  )
        KRATOS_REGISTER_VARIABLE(  MAX_VEL  )
        KRATOS_REGISTER_VARIABLE( FRONT_MEETING)
		KRATOS_REGISTER_VARIABLE( LIQUID_TIME)
		KRATOS_REGISTER_VARIABLE( FLOW_LENGTH )
		KRATOS_REGISTER_VARIABLE( FLOW_LENGTH2 )
		KRATOS_REGISTER_VARIABLE( COOLING_RATE)
		KRATOS_REGISTER_VARIABLE (LIQUID_TO_SOLID_TIME)
		KRATOS_REGISTER_VARIABLE (TIME_CRT)
		KRATOS_REGISTER_VARIABLE(SINKMARK)
		KRATOS_REGISTER_VARIABLE(COLD_SHUTS)
        KRATOS_REGISTER_VARIABLE(NIYAMA)
        KRATOS_REGISTER_VARIABLE(INITIAL_TEMPERATURE)
        KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS(TEMPERATURE_GRADIENT)
  }


}  // namespace Kratos.
