//    |  /           |
//    ' /   __| _` | __|  _ \   __|
//    . \  |   (   | |   (   |\__ \.
//   _|\_\_|  \__,_|\__|\___/ ____/
//                   Multi-Physics
//
//  License:		 BSD License
//					 Kratos default license: kratos/license.txt
//
//  Main authors:    Riccardo Rossi
//

// System includes
#include <string>
#include <iostream>
#include <vector>

// External includes

// Project includes
#include "includes/define.h"
#include "includes/dem_variables.h"
#include "includes/kernel.h"
#include "includes/node.h"

#include "includes/kratos_flags.h"

namespace Kratos
{

    KRATOS_CREATE_VARIABLE( int, PARTICLE_MATERIAL )
    KRATOS_CREATE_VARIABLE(int, NUMBER_OF_INIT_BASSET_STEPS)
    KRATOS_CREATE_VARIABLE( int, FRAME_OF_REFERENCE_TYPE )
    KRATOS_CREATE_VARIABLE( double, PARTICLE_MASS )
    KRATOS_CREATE_VARIABLE( double, RADIUS )
    KRATOS_CREATE_VARIABLE( double, SEARCH_RADIUS_INCREMENT )
    KRATOS_CREATE_VARIABLE( double, SEARCH_RADIUS_INCREMENT_FOR_WALLS )
    KRATOS_CREATE_VARIABLE( double, DEM_DELTA_TIME )
    KRATOS_CREATE_VARIABLE( int, FIXED_MESH_OPTION )
    KRATOS_CREATE_VARIABLE( int, PARTITION_MASK )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS(TOTAL_FORCES)
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( DAMP_FORCES )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( AUX_VEL )
    KRATOS_CREATE_VARIABLE( Vector, NEIGHBOURS_IDS_DOUBLE )
    KRATOS_CREATE_VARIABLE( DenseVector<double>, BASSET_HISTORIC_INTEGRANDS )
    KRATOS_CREATE_VARIABLE( DenseVector<double>, HINSBERG_TAIL_CONTRIBUTIONS )
    KRATOS_CREATE_VARIABLE( Vector, PARTICLE_ROTATE_SPRING_FAILURE_TYPE )
    KRATOS_CREATE_VARIABLE( DenseVector<int>, OLD_NEIGHBOURS_IDS )
    KRATOS_CREATE_VARIABLE( DenseVector<int>, INI_NEIGHBOURS_IDS )
    KRATOS_CREATE_VARIABLE( DenseVector<int>, CONTINUUM_INI_NEIGHBOURS_IDS )
    KRATOS_CREATE_VARIABLE( DenseVector<int>, NEIGHBOURS_IDS )
    KRATOS_CREATE_VARIABLE( DenseVector<int>, PARTICLE_INITIAL_FAILURE_ID )
    KRATOS_CREATE_VARIABLE( DenseVector<int>, CONTINUUM_PARTICLE_INITIAL_FAILURE_ID )
    KRATOS_CREATE_VARIABLE( std::string, ELEMENT_TYPE)
    KRATOS_CREATE_VARIABLE( VectorArray3Double, PARTICLE_ROTATE_SPRING_MOMENT )
    KRATOS_CREATE_VARIABLE( int, COUPLING_TYPE)
    KRATOS_CREATE_VARIABLE( int, NON_NEWTONIAN_OPTION )
    KRATOS_CREATE_VARIABLE( int, MANUALLY_IMPOSED_DRAG_LAW_OPTION )
    KRATOS_CREATE_VARIABLE( int, DRAG_MODIFIER_TYPE )
    KRATOS_CREATE_VARIABLE( int, BUOYANCY_FORCE_TYPE )
    KRATOS_CREATE_VARIABLE( int, DRAG_FORCE_TYPE )
    KRATOS_CREATE_VARIABLE( int, VIRTUAL_MASS_FORCE_TYPE )
    KRATOS_CREATE_VARIABLE( int, BASSET_FORCE_TYPE )
    KRATOS_CREATE_VARIABLE( int, LIFT_FORCE_TYPE )
    KRATOS_CREATE_VARIABLE( int, MAGNUS_FORCE_TYPE )
    KRATOS_CREATE_VARIABLE( int, HYDRO_TORQUE_TYPE )
    KRATOS_CREATE_VARIABLE( int, FLUID_MODEL_TYPE )
    KRATOS_CREATE_VARIABLE( int, DRAG_POROSITY_CORRECTION_TYPE )
    KRATOS_CREATE_VARIABLE( int, TIME_STEPS_PER_QUADRATURE_STEP )
    KRATOS_CREATE_VARIABLE( int, QUADRATURE_ORDER )
    KRATOS_CREATE_VARIABLE( int, CURRENT_COMPONENT )
    KRATOS_CREATE_VARIABLE( double, POWER_LAW_TOLERANCE )
    KRATOS_CREATE_VARIABLE( double, PARTICLE_SPHERICITY )
    KRATOS_CREATE_VARIABLE( double, INIT_DRAG_FORCE )
    KRATOS_CREATE_VARIABLE( double, DRAG_LAW_SLOPE )
    KRATOS_CREATE_VARIABLE( double, DISPERSE_FRACTION )
    KRATOS_CREATE_VARIABLE( double, DISPERSE_FRACTION_RATE )
    KRATOS_CREATE_VARIABLE( double, FLUID_FRACTION )
    KRATOS_CREATE_VARIABLE( double, FLUID_FRACTION_OLD )
    KRATOS_CREATE_VARIABLE( double, FLUID_FRACTION_RATE )
    KRATOS_CREATE_VARIABLE( double, FLUID_FRACTION_FILTERED )
    KRATOS_CREATE_VARIABLE( double, PHASE_FRACTION )
    KRATOS_CREATE_VARIABLE( double, PHASE_FRACTION_RATE )
    KRATOS_CREATE_VARIABLE( double, DISPERSE_FRACTION_PROJECTED )
    KRATOS_CREATE_VARIABLE( double, FLUID_FRACTION_PROJECTED )
    KRATOS_CREATE_VARIABLE( double, FLUID_DENSITY_PROJECTED )
    KRATOS_CREATE_VARIABLE( double, FLUID_VISCOSITY_PROJECTED )
    KRATOS_CREATE_VARIABLE( double, REYNOLDS_NUMBER )
    KRATOS_CREATE_VARIABLE( double, DRAG_COEFFICIENT )
    KRATOS_CREATE_VARIABLE( double, SHEAR_RATE_PROJECTED )
    KRATOS_CREATE_VARIABLE( double, DELTA_TIME_QUADRATURE )
    KRATOS_CREATE_VARIABLE( double, LAST_TIME_APPENDING )
    KRATOS_CREATE_VARIABLE( double, TIME_AVERAGED_DOUBLE )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( VELOCITY_OLD )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( VELOCITY_OLD_OLD )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( ACCELERATION_MOVING_FRAME_ORIGIN )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( ANGULAR_VELOCITY_MOVING_FRAME )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( ANGULAR_VELOCITY_MOVING_FRAME_OLD )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( ANGULAR_ACCELERATION_MOVING_FRAME )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( SLIP_VELOCITY )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( ADDITIONAL_FORCE )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( ADDITIONAL_FORCE_OLD )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( ADDITIONAL_FORCE_OLD_OLD )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( HYDRODYNAMIC_FORCE )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( HYDRODYNAMIC_MOMENT )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( FLUID_VEL_PROJECTED )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( FLUID_VEL_PROJECTED_RATE )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( FLUID_VEL_LAPL_PROJECTED )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( FLUID_VEL_LAPL_RATE_PROJECTED )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( FLUID_ACCEL_PROJECTED )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( FLUID_ACCEL_FOLLOWING_PARTICLE_PROJECTED )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( MATERIAL_FLUID_ACCEL_PROJECTED )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( FLUID_VORTICITY_PROJECTED )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( PARTICLE_VEL_FILTERED )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( HYDRODYNAMIC_REACTION )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( MEAN_HYDRODYNAMIC_REACTION )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( DRAG_REACTION )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( LIFT_FORCE )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( VIRTUAL_MASS_FORCE )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( BASSET_FORCE )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( EXTERNAL_APPLIED_FORCE )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( EXTERNAL_APPLIED_MOMENT )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( BUOYANCY )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( PRESSURE_GRADIENT )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( PRESSURE_GRAD_PROJECTED )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( DISPERSE_FRACTION_GRADIENT )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( DISPERSE_FRACTION_GRADIENT_PROJECTED )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( FLUID_FRACTION_GRADIENT )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( FLUID_FRACTION_GRADIENT_PROJECTED )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( PHASE_FRACTION_GRADIENT )
    KRATOS_CREATE_3D_VARIABLE_WITH_COMPONENTS( TIME_AVERAGED_ARRAY_3 )

    // Nano-particle only BEGIN
    KRATOS_CREATE_VARIABLE( double, CATION_CONCENTRATION )
    // Nano-particle only END

    //SINTERING
    KRATOS_CREATE_VARIABLE( double, ATOMIC_VOLUME )
    KRATOS_CREATE_VARIABLE( double, SURFACE_ENERGY )
    KRATOS_CREATE_VARIABLE( double, DIHEDRAL_ANGLE )
    KRATOS_CREATE_VARIABLE( double, SINTERING_START_TEMPERATURE )
    KRATOS_CREATE_VARIABLE( double, RELAXATION_TIME )
    KRATOS_CREATE_VARIABLE( double, LARGE_VISCOSITY_COEFFICIENT )
    KRATOS_CREATE_VARIABLE( double, PRE_EXP_DIFFUSION_COEFFICIENT )
    KRATOS_CREATE_VARIABLE( double, GB_WIDTH )
    KRATOS_CREATE_VARIABLE( double, ENTHAPLY_ACTIVATION )
    //

  void KratosApplication::RegisterDEMVariables()
  {

    //--------------- DEM Application -------------------//
    KRATOS_REGISTER_VARIABLE( FRAME_OF_REFERENCE_TYPE )
    KRATOS_REGISTER_VARIABLE( PARTICLE_MASS )
    KRATOS_REGISTER_VARIABLE( RADIUS )
    KRATOS_REGISTER_VARIABLE( SEARCH_RADIUS_INCREMENT )
    KRATOS_REGISTER_VARIABLE( SEARCH_RADIUS_INCREMENT_FOR_WALLS )
    KRATOS_REGISTER_VARIABLE( PARTICLE_MATERIAL )
    KRATOS_REGISTER_VARIABLE( DEM_DELTA_TIME )
    KRATOS_REGISTER_VARIABLE( FIXED_MESH_OPTION )
    KRATOS_REGISTER_VARIABLE( PARTITION_MASK )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( TOTAL_FORCES )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( DAMP_FORCES )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( AUX_VEL )
    KRATOS_REGISTER_VARIABLE( NEIGHBOURS_IDS_DOUBLE )
    KRATOS_REGISTER_VARIABLE( PARTICLE_ROTATE_SPRING_FAILURE_TYPE )
    KRATOS_REGISTER_VARIABLE( OLD_NEIGHBOURS_IDS )
    KRATOS_REGISTER_VARIABLE( INI_NEIGHBOURS_IDS )
    KRATOS_REGISTER_VARIABLE( CONTINUUM_INI_NEIGHBOURS_IDS )
    KRATOS_REGISTER_VARIABLE( NEIGHBOURS_IDS )
    KRATOS_REGISTER_VARIABLE( PARTICLE_INITIAL_FAILURE_ID )
    KRATOS_REGISTER_VARIABLE( CONTINUUM_PARTICLE_INITIAL_FAILURE_ID )
    KRATOS_REGISTER_VARIABLE( ELEMENT_TYPE )
    KRATOS_REGISTER_VARIABLE( PARTICLE_ROTATE_SPRING_MOMENT )

    // Swimming DEM Application BEGINNING
    KRATOS_REGISTER_VARIABLE( NUMBER_OF_INIT_BASSET_STEPS )
    KRATOS_REGISTER_VARIABLE( COUPLING_TYPE)
    KRATOS_REGISTER_VARIABLE( NON_NEWTONIAN_OPTION )
    KRATOS_REGISTER_VARIABLE( MANUALLY_IMPOSED_DRAG_LAW_OPTION )
    KRATOS_REGISTER_VARIABLE( DRAG_MODIFIER_TYPE )
    KRATOS_REGISTER_VARIABLE( BUOYANCY_FORCE_TYPE )
    KRATOS_REGISTER_VARIABLE( DRAG_FORCE_TYPE )
    KRATOS_REGISTER_VARIABLE( VIRTUAL_MASS_FORCE_TYPE )
    KRATOS_REGISTER_VARIABLE( BASSET_FORCE_TYPE )
    KRATOS_REGISTER_VARIABLE( LIFT_FORCE_TYPE )
    KRATOS_REGISTER_VARIABLE( MAGNUS_FORCE_TYPE )
    KRATOS_REGISTER_VARIABLE( HYDRO_TORQUE_TYPE )
    KRATOS_REGISTER_VARIABLE( FLUID_MODEL_TYPE )
    KRATOS_REGISTER_VARIABLE( DRAG_POROSITY_CORRECTION_TYPE )
    KRATOS_REGISTER_VARIABLE( TIME_STEPS_PER_QUADRATURE_STEP )
    KRATOS_REGISTER_VARIABLE( QUADRATURE_ORDER )
    KRATOS_REGISTER_VARIABLE( CURRENT_COMPONENT )
    KRATOS_REGISTER_VARIABLE( POWER_LAW_TOLERANCE )
    KRATOS_REGISTER_VARIABLE( PARTICLE_SPHERICITY )
    KRATOS_REGISTER_VARIABLE( INIT_DRAG_FORCE )
    KRATOS_REGISTER_VARIABLE( DRAG_LAW_SLOPE )
    KRATOS_REGISTER_VARIABLE( DISPERSE_FRACTION )
    KRATOS_REGISTER_VARIABLE( DISPERSE_FRACTION_RATE )
    KRATOS_REGISTER_VARIABLE( FLUID_FRACTION )
    KRATOS_REGISTER_VARIABLE( FLUID_FRACTION_OLD )
    KRATOS_REGISTER_VARIABLE( FLUID_FRACTION_RATE )
    KRATOS_REGISTER_VARIABLE( FLUID_FRACTION_FILTERED )
    KRATOS_REGISTER_VARIABLE( PHASE_FRACTION )
    KRATOS_REGISTER_VARIABLE( PHASE_FRACTION_RATE )
    KRATOS_REGISTER_VARIABLE( DISPERSE_FRACTION_PROJECTED )
    KRATOS_REGISTER_VARIABLE( FLUID_FRACTION_PROJECTED )
    KRATOS_REGISTER_VARIABLE( FLUID_DENSITY_PROJECTED )
    KRATOS_REGISTER_VARIABLE( FLUID_VISCOSITY_PROJECTED )
    KRATOS_REGISTER_VARIABLE( REYNOLDS_NUMBER )
    KRATOS_REGISTER_VARIABLE( DRAG_COEFFICIENT )
    KRATOS_REGISTER_VARIABLE( SHEAR_RATE_PROJECTED )
    KRATOS_REGISTER_VARIABLE( DELTA_TIME_QUADRATURE )
    KRATOS_REGISTER_VARIABLE( LAST_TIME_APPENDING )
    KRATOS_REGISTER_VARIABLE( TIME_AVERAGED_DOUBLE )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( VELOCITY_OLD )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( VELOCITY_OLD_OLD )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( ACCELERATION_MOVING_FRAME_ORIGIN )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( ANGULAR_VELOCITY_MOVING_FRAME )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( ANGULAR_VELOCITY_MOVING_FRAME_OLD )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( ANGULAR_ACCELERATION_MOVING_FRAME )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( SLIP_VELOCITY )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( ADDITIONAL_FORCE )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( ADDITIONAL_FORCE_OLD )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( ADDITIONAL_FORCE_OLD_OLD )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( HYDRODYNAMIC_FORCE )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( HYDRODYNAMIC_MOMENT )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( FLUID_VEL_PROJECTED )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( FLUID_VEL_PROJECTED_RATE )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( FLUID_VEL_LAPL_PROJECTED )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( FLUID_VEL_LAPL_RATE_PROJECTED )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( FLUID_ACCEL_PROJECTED )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( FLUID_ACCEL_FOLLOWING_PARTICLE_PROJECTED )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( MATERIAL_FLUID_ACCEL_PROJECTED )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( FLUID_VORTICITY_PROJECTED )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( PARTICLE_VEL_FILTERED )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( HYDRODYNAMIC_REACTION )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( MEAN_HYDRODYNAMIC_REACTION )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( DRAG_REACTION )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( LIFT_FORCE )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( VIRTUAL_MASS_FORCE )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( BASSET_FORCE )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( BUOYANCY )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( EXTERNAL_APPLIED_FORCE )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( EXTERNAL_APPLIED_MOMENT )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( PRESSURE_GRADIENT )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( PRESSURE_GRAD_PROJECTED )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( DISPERSE_FRACTION_GRADIENT )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( DISPERSE_FRACTION_GRADIENT_PROJECTED )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( FLUID_FRACTION_GRADIENT )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( FLUID_FRACTION_GRADIENT_PROJECTED )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( PHASE_FRACTION_GRADIENT )
    KRATOS_REGISTER_3D_VARIABLE_WITH_COMPONENTS( TIME_AVERAGED_ARRAY_3 )

    KRATOS_REGISTER_VARIABLE( BASSET_HISTORIC_INTEGRANDS )
    KRATOS_REGISTER_VARIABLE( HINSBERG_TAIL_CONTRIBUTIONS )
    // Swimming DEM Application END

    // Nano-particle only BEGIN
    KRATOS_REGISTER_VARIABLE( CATION_CONCENTRATION )
    // Nano-particle only END

    // DEM Sintering
    KRATOS_REGISTER_VARIABLE( ATOMIC_VOLUME )
    KRATOS_REGISTER_VARIABLE( SURFACE_ENERGY )
    KRATOS_REGISTER_VARIABLE( DIHEDRAL_ANGLE )
    KRATOS_REGISTER_VARIABLE( SINTERING_START_TEMPERATURE )
    KRATOS_REGISTER_VARIABLE( RELAXATION_TIME )
    KRATOS_REGISTER_VARIABLE( LARGE_VISCOSITY_COEFFICIENT )
    KRATOS_REGISTER_VARIABLE( PRE_EXP_DIFFUSION_COEFFICIENT )
    KRATOS_REGISTER_VARIABLE( GB_WIDTH )
    KRATOS_REGISTER_VARIABLE( ENTHAPLY_ACTIVATION )

  }


}  // namespace Kratos.
